@@ -1,11 +1,11 @@
 package Set::Scalar;
 
 use strict;
-local $^W = 1;
+# local $^W = 1;
 
 use vars qw($VERSION @ISA);
 
-$VERSION = '1.19';
+$VERSION = '1.21';
 
 @ISA = qw(Set::Scalar::Real Set::Scalar::Null Set::Scalar::Base);
 
@@ -279,6 +279,12 @@ The first priority of Set::Scalar is to be a convenient interface to sets.
 While not designed to be slow or big, neither has it been designed to
 be fast or compact.
 
+Using references (or objects) as set members has not been extensively
+tested.  The desired semantics are not always clear: what should
+happen when the elements behind the references change? Especially
+unclear is what should happen when the objects start having their
+own stringification overloads.
+
 =head1 SEE ALSO
 
 Set::Bag for bags (multisets, counted sets), and Bit::Vector for fast
