@@ -11,6 +11,35 @@ use vars qw(@ISA @EXPORT_OK);
 
 use UNIVERSAL 'isa';
 
+BEGIN {
+    eval 'require Scalar::Util';
+    unless ($@) {
+	import Scalar::Util qw(blessed refaddr);
+    } else {
+	# Use the pure Perl emulations (directly snagged from Scalar::Util).
+	sub UNIVERSAL::a_sub_not_likely_to_be_here { ref($_[0]) }
+	*blessed = sub ($) {
+	    local($@, $SIG{__DIE__}, $SIG{__WARN__});
+	    length(ref($_[0]))
+		? eval { $_[0]->a_sub_not_likely_to_be_here }
+	    : undef
+	};
+        *refaddr = sub ($) {
+	    my $pkg = ref($_[0]) or return undef;
+	    if (blessed($_[0])) {
+		bless $_[0], 'Scalar::Util::Fake';
+	    }
+	    else {
+		$pkg = undef;
+	    }
+	    "$_[0]" =~ /0x(\w+)/;
+	    my $i = do { local $^W; hex $1 };
+	    bless $_[0], $pkg if defined $pkg;
+	    $i;
+	};
+    }
+}
+
 @EXPORT_OK = qw(_make_elements
 		as_string
 		as_string_callback
@@ -77,10 +106,7 @@ sub new {
 sub _strval {
     my $class = ref $_[0];
     return $_[0] unless $class;
-    bless $_[0], 'Set::Scalar::Base::NonOverloaded';
-    my $strval = "$class($_[0])";
-    bless $_[0], $class;
-    return $strval;
+    sprintf "%s(%s)", $class, refaddr $_[0];
 }
 
 sub _make_elements {
@@ -239,7 +265,7 @@ sub union {
     }
 
     $union = $self
-	if $union->size == $self->size;
+	if $is_universal && $union->size == $self->size;
 
     return $union;
 }
@@ -284,17 +310,18 @@ sub intersection {
 
     my $intersection = $self->clone;
 
+    my $is_null;
+
     foreach my $next ( @_ ) {
 	unless ($next->is_universal) {
-	    ($intersection, my $is_null) =
-		$intersection->_intersection( $next );
+	    ($intersection, $is_null) =	$intersection->_intersection( $next );
 
 	    last if $is_null;
 	}
     }
 
     $intersection = $self
-	if $intersection->size == $self->size;
+	if $is_null && $intersection->size == $self->size;
 
     return $intersection;
 }
@@ -599,7 +626,7 @@ sub _elements_as_string {
 	    } else {
 		$recursive = RECURSIVE_DEEP;
 	    }
-	} elsif (ref $element && $element->isa(__PACKAGE__)) {
+	} elsif (blessed $element && $element->isa(__PACKAGE__)) {
 	    local $history{ $element_id } = 1;
 	    push @complex_elements, $element->as_string( \%history );
 	} else {
